package MineSweeper;

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import my.components.PButton;

public class MineButton extends PButton {

  // Constants
  public static final String FLAG = "F";
  public static final String FREE = " ";
  public static final String MINE = "\u00D7";
  public static final String WRONG = "W";
  public static final String ZERO = "  ";
  public static final MineButton NO_BUTTON = null;

  protected static MineButton current_button = NO_BUTTON;

  private boolean is_mine;
  private static final Color BG = Color.lightGray;
  private Point id;

  // The constructors
  public MineButton(Point new_id, boolean mine) {
    super(FREE);
    square = true;
    highlight = true;
    setBackground(BG);
    id = new_id;
    is_mine = mine;
  }
  public MineButton(Point new_id, boolean mine, int dim) {
    super(FREE);
    forceDimension(new Dimension(dim,dim));
    highlight = true;
    setBackground(BG);
    id = new_id;
    is_mine = mine;
  }
  public MineButton(Point new_id, boolean mine, Font new_font) {
    super(FREE, new_font);
    square = true;
    highlight = true;
    setBackground(BG);
    id = new_id;
    is_mine = mine;
  }
  public MineButton(Point new_id, boolean mine, int dim, Font new_font) {
    super(FREE, new_font);
    forceDimension(new Dimension(dim,dim));
    highlight = true;
    setBackground(BG);
    id = new_id;
    is_mine = mine;
  }
  public MineButton(Point new_id, boolean mine, Font new_font, int margin_dim) {
    super(FREE, new_font, margin_dim, margin_dim);
    square = true;
    highlight = true;
    setBackground(BG);
    id = new_id;
    is_mine = mine;
  }

  // This method flattens the button
  public void Done() {
    flatten=true;
    highlight=false;
    paint(getGraphics());
  }


  // This method changes the value of has_mine
  public void Reset(boolean mine) {
    is_mine = mine;
    if (!IsFree()) {
      flatten=false;
      highlight=true;
      SetFree();
    }
  }


  //**************************************************
  // These methods return a simple boolean value of
  // what the square is (FLAG, FREE, NUMBER)
  //**************************************************
  public boolean IsFlag() {
    return getLabel() == FLAG;
  }

  public boolean IsFree() {
    return getLabel() == FREE;
  }

  public boolean IsHiddenMine() {
    return is_mine;
  }

  public boolean IsMine() {
    return getLabel() == MINE;
  }

  public boolean IsNumber() {
    return flatten;
  }

  public boolean IsWrong() {
    return getLabel() == WRONG;
  }

  public int GetNumber() {
    if (getLabel() == ZERO)
      return 0;
    return Integer.parseInt(getLabel());
  }


  //***************************************************
  // These methods ease the changing of square names
  //***************************************************
  public void SetFlag(Image image) {
    //super.setBackground(addGreen(BG, 0.10f));
    setLabel(FLAG);
    setImage(image);
    paint(getGraphics());
  }

  public void SetFree() {
    setBackground(BG);
    setLabel(FREE);
    paint(getGraphics());
  }

  public void SetMine(Image image) {
    setLabel(MINE);
    setImage(image);
    paint(getGraphics());
  }

  public void SetNumber(int new_label) {
    flatten = true;
    highlight = false;
    if (new_label == 0)
      setLabel(ZERO);
    else
      setLabel(String.valueOf(new_label));
    paint(getGraphics());
  }

  public void SetWrong(Image image) {
    setLabel(WRONG);
    setImage(image);
    paint(getGraphics());
  }


  // These methods simplify the modification of colors
  public static Color addRed(Color old_color, float percent_change) {
    int change = (int)(255f * percent_change);
    int 
      r = old_color.getRed() + change, 
      g = old_color.getGreen() - change, 
      b = old_color.getBlue() - change;

    if (r > 255) r = 255;
    if (g < 0) g = 0;
    if (b < 0) b = 0;
    
    return new Color(r, g, b);
  }

  public static Color addGreen(Color old_color, float percent_change) {
    int change = (int)(255f * percent_change);
    int 
      r = old_color.getRed() - change, 
      g = old_color.getGreen() + change, 
      b = old_color.getBlue() - change;

    if (r < 0) r = 0;
    if (g > 255) g = 255;
    if (b < 0) b = 0;
    
    return new Color(r, g, b);
  }

  public static Color addBlue(Color old_color, float percent_change) {
    int change = (int)(255f * percent_change);
    int 
      r = old_color.getRed() - change, 
      g = old_color.getGreen() - change, 
      b = old_color.getBlue() + change;

    if (r < 0) r = 0;
    if (g < 0) g = 0;
    if (b > 255) b = 255;
    
    return new Color(r, g, b);
  }

  // These methods provide hard highlighting capabilities
  public void highlight() {
    super.mouse_inside = true;
    super.paint(getGraphics());
  }

  public void unhighlight() {
    super.mouse_inside = false;
    super.paint(getGraphics());
  }

  // These methods update the current_id variable.
  public void mouseEntered(MouseEvent e) {
    current_button = this;
    super.mouseEntered(e);
  }

  public void mouseExited(MouseEvent e) {
    current_button = NO_BUTTON;
    super.mouseExited(e);
  }

  public Point GetID() {
    //return new Point(id.x, id.y);
    return id;
  }

  public static MineButton CurrentButton() {
    return current_button;
  }

}
