/*
 *  Welcome to MineSweeper.
 *
 *  This file contains the code to the actual applet.  It handles any
 *  parameters, file reading, etc.  It also assembles the different
 *  components of the applet into something sensible.  
 *
 *  I make no guarantees about this code.  It may blow up your computer, 
 *  swear at you, or have an inconsistent style.
 */
package MineSweeper;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import my.components.*;

public class MineSweeper 
  extends Applet 
  implements ActionListener, MineFieldListener, MouseListener, Runnable 
{

  // Components & information
  private ImageRow flags_left;
  private PButton new_game;
  private MineField mine_field;

  private Vector numbers;
  private Vector flags_left_vect;

  // Timer specific variables
  private Thread timer_thread;
  private Date game_time;
  private ImageRow timer;
  private Vector timer_vect;
  private Image colon;

  // Mine field information
  private Dimension field_dim = new Dimension(0, 0);
  private float mine_ratio = 0.10f;
  private Image flag = null;
  private Image mine = null;
  private Image mineX = null;
  private static final int mine_dim = 23;

  // New game button faces
  private Image face_smile;
  private Image face_ooo;
  private Image face_dead;
  private Image face_cool;

  protected String VERSION = new String("v1.0");

  public void init() {

    super.init();
    addMouseListener(this);
    setBackground(Color.lightGray);

    // Gather the parameters
    String param = getParameter("percent");
    if (param != null) {
      try {
	Float new_float = new Float(param);
	mine_ratio = new_float.floatValue();
      }
      catch (NumberFormatException e) {}
    }

    // Load the graphics
    showStatus("Loading graphics...");
    {
      MediaTracker tracker = new MediaTracker(this);
      Image number;

      // load numbers
      numbers = new Vector(10);
      for (int i = 0; i < 10; ++i) {
	number = getImage(getDocumentBase(), i + ".gif");
	tracker.addImage(number, i);
	numbers.addElement(number);
      }
      colon = getImage(getDocumentBase(), "colon.gif");
      tracker.addImage(colon, 10);

      // load other graphics
      flag = getImage(getDocumentBase(), "flag.gif");
      tracker.addImage(flag, 11);
      mine = getImage(getDocumentBase(), "mine.gif");
      tracker.addImage(mine, 12);
      mineX = getImage(getDocumentBase(), "mineX.gif");
      tracker.addImage(mineX, 13);
      face_smile = getImage(getDocumentBase(), "smile.gif");
      tracker.addImage(face_smile, 14);
      face_ooo = getImage(getDocumentBase(), "ooo.gif");
      tracker.addImage(face_ooo, 15);
      face_dead = getImage(getDocumentBase(), "dead.gif");
      tracker.addImage(face_dead, 16);
      face_cool = getImage(getDocumentBase(), "cool.gif");
      tracker.addImage(face_cool, 17);

      // finish loading and check
      try {tracker.waitForAll();}
      catch (InterruptedException e) {}
      for (int i = 0; i < 10; ++i)
	if (tracker.isErrorID(i))
	  System.err.println("Error loading `" + i + ".gif'");
      if (tracker.isErrorID(10))
      System.err.println("Error loading `colon.gif'");
      if (tracker.isErrorID(11))
	System.err.println("Error loading `flag.gif'");
      if (tracker.isErrorID(12))
	System.err.println("Error loading `mine.gif'");
      if (tracker.isErrorID(13))
	System.err.println("Error loading `mineX.gif'");
      if (tracker.isErrorID(14))
	System.err.println("Error loading `smile.gif'");
      if (tracker.isErrorID(15))
	System.err.println("Error loading `ooo.gif'");
      if (tracker.isErrorID(16))
	System.err.println("Error loading `dead.gif'");
      if (tracker.isErrorID(17))
	System.err.println("Error loading `cool.gif'");
    }
    showStatus("");

    setLayout(new GridBagLayout());

    // Create the tool bar
    Panel tools = new Panel();
    tools.setLayout(new GridBagLayout());

    // Initialize the flags_left counter
    flags_left_vect = new Vector(3);
    flags_left_vect.addElement(numbers.elementAt(0));
    flags_left_vect.addElement(numbers.elementAt(0));
    flags_left_vect.addElement(numbers.elementAt(0));
    flags_left = new ImageRow(flags_left_vect, ImageRow.HORIZONTAL, ImageRow.CENTER);
    flags_left.setBackground(Color.black);

    // Initialize the new game button
    new_game = new PButton(face_smile);
    new_game.addActionListener(this);

    // Initialize the timer
    timer_vect = new Vector(8);
    timer_vect.addElement(numbers.elementAt(0));
    timer_vect.addElement(numbers.elementAt(0));
    timer_vect.addElement(colon);
    timer_vect.addElement(numbers.elementAt(0));
    timer_vect.addElement(numbers.elementAt(0));
    timer_vect.addElement(colon);
    timer_vect.addElement(numbers.elementAt(0));
    timer_vect.addElement(numbers.elementAt(0));
    timer = new ImageRow(timer_vect, ImageRow.HORIZONTAL, ImageRow.CENTER);
    timer.setBackground(Color.black);

    int padx = (getSize().width - 3 * timer.getPreferredSize().width) / 2;
    constrain(tools, flags_left, 
	      0, 0, 1, 1, 
	      GridBagConstraints.NONE, GridBagConstraints.WEST, 
	      0.0, 0.0, 0, 0, 0, (timer.getPreferredSize().width - 
				  flags_left.getPreferredSize().width));
    constrain(tools, new_game, 
	      1, 0, 1, 1, 
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 
	      0.0, 0.0, 5, padx, 0, padx);
    constrain(tools, timer, 
	      2, 0, 1, 1, 
	      GridBagConstraints.NONE, GridBagConstraints.EAST, 
	      0.0, 0.0, 0, 0, 0, 0);

    constrain(this, tools, 
	      0, 1, 1, 1, 
	      GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 
	      0.0, 0.0, 0, 0, 0, 0);

    // This little diddy, tries to size the mine field to the best of its ability
    showStatus("Creating minefield...");
    field_dim.width = getSize().width / mine_dim;
    field_dim.height = (getSize().height - tools.getPreferredSize().height - 
			10) / mine_dim;
    if (field_dim.width < 0)  field_dim.width = 0;
    if (field_dim.height < 0) field_dim.height = 0;

    // Okay, create the mine field
    Font font = new Font("Helvetica", Font.BOLD, 16);
    mine_field = new MineField(field_dim, mine_dim, mine_ratio, font, flag, 
			       mine, mineX, this);
    mine_field.ClickFree();
    NewMineCount(mine_field.GetMineCount());

    // Add the minefield to the applet
    constrain(this, mine_field, 
	      0, 2, 1, 1,  
	      GridBagConstraints.NONE, GridBagConstraints.CENTER, 
	      1.0, 1.0, 5, 0, 0, 0);
    showStatus("");

    // Initialize the thread
    game_time = new Date();
    timer_thread = new Thread(this);
    timer_thread.start();
  }

  public void constrain(Container container, Component component,
			int grid_x, int grid_y, int grid_width, int grid_height,
			int fill, int anchor, double weight_x, double weight_y, 
			int top, int left, int bottom, int right) {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx = grid_x; c.gridy = grid_y;
    c.gridwidth = grid_width; c.gridheight = grid_height;
    c.fill = fill; c.anchor = anchor;
    c.weightx = weight_x; c.weighty = weight_y;
    if (top+bottom+left+right > 0)
      c.insets = new Insets(top, left, bottom, right);
    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }

  public void constrain(Container container, Component component,
			int grid_x, int grid_y, int grid_width, int grid_height) {
    constrain(container, component, grid_x, grid_y, grid_width, grid_height, GridBagConstraints.NONE,
	      GridBagConstraints.NORTHWEST, 0.0, 0.0, 0, 0, 0, 0);
  }

  public void constrain(Container container, Component component,
			int grid_x, int grid_y, int grid_width, int grid_height,
			int top, int left, int bottom, int right) {
    constrain(container, component, grid_x, grid_y, grid_width, grid_height, GridBagConstraints.NONE,
	      GridBagConstraints.NORTHWEST, 
	      0.0, 0.0, top, left, bottom, right);
  }


  // This method simply records where the user last pressed the mouse button
  // down.  It will be used later to see if the button should be revealed.
  // It also records which button was pressed
  public void mousePressed(MouseEvent e) {
  }


  // This method handles the mine field's reaction to the user pressing
  // and releasing the mouse button over a button in the field
  public void mouseReleased(MouseEvent e) {
  }

  // These methods display a better message in the status bar
  public void mouseEntered(MouseEvent e) {
    showStatus("MineSweeper " + VERSION);
  }

  public void mouseExited(MouseEvent e) {
    showStatus("");
  }

  // This method just completes the MouseListener requirements
  public void mouseClicked(MouseEvent e) {
  }

  // This method handles the action events for the game
  public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand() == new_game.getLabel()) {
      new_game.setImage(face_smile);
      new_game.paint(new_game.getGraphics());
      mine_field.ResetField();
      mine_field.ClickFree();
      game_time = new Date();
      timer_thread.resume();
    }
  }


  /*
   *  MousePressed is called when the player presses the button in the mine field
   */
  public void MousePressed() {
    new_game.setImage(face_ooo);
    new_game.paint(new_game.getGraphics());
  }


  /*
   *  MouseReleased is called when the player releases the mouse button from the 
   *  mine field
   */
  public void MouseReleased() {
    new_game.setImage(face_smile);
    new_game.paint(new_game.getGraphics());
  }


  // This method is called whenever the minefield changes the mine count
  public void NewMineCount(int mine_count) {
    flags_left_vect.setElementAt(numbers.elementAt(mine_count/100), 0);
    flags_left_vect.setElementAt(numbers.elementAt((mine_count%100)/10), 1);
    flags_left_vect.setElementAt(numbers.elementAt(mine_count%10), 2);
    flags_left.repaint();
  }


  // This method is called whenever the minefield changes the number of 
  // remaining squares
  public void NewSquaresLeft(int squares_left) {
    if (squares_left == 0) {
      new_game.setImage(face_cool);
      new_game.paint(new_game.getGraphics());
    }
  }


  /*
   *  UserDead is called when the player hits a mine
   */
  public void UserDead() {
    new_game.setImage(face_dead);
    new_game.paint(new_game.getGraphics());
  }


  // This method is called whenever the minefield locks out the user
  public void UserLocked() {
    timer_thread.suspend();
  }


  /*
   *  run implements a basic timer for the application
   */
  public void run() {
    int hours, minutes, seconds;
    long diff_time;
    while (true) {
      diff_time = ((new Date()).getTime() - game_time.getTime()) / 1000;
      seconds = (int)(diff_time % 60);
      diff_time /= 60;
      minutes = (int)(diff_time % 60);
      diff_time /= 60;
      hours = (int)(diff_time % 24);

      // Change the numbers in the vector to reflect the current game time
      timer_vect.setElementAt(numbers.elementAt(hours / 10), 0);
      timer_vect.setElementAt(numbers.elementAt(hours % 10), 1);
      timer_vect.setElementAt(numbers.elementAt(minutes / 10), 3);
      timer_vect.setElementAt(numbers.elementAt(minutes % 10), 4);
      timer_vect.setElementAt(numbers.elementAt(seconds / 10), 6);
      timer_vect.setElementAt(numbers.elementAt(seconds % 10), 7);
      timer.repaint();

      try {
	timer_thread.sleep(1000);
      }
      catch (InterruptedException e) {}
    }
  }
}
