package my.components;

import java.awt.*;
import java.util.*;

public class ImageRow extends Component {

  // Constants
  public static final int HORIZONTAL = 1;
  public static final int VERTICAL   = 2;
  public static final int NORTH  = 1;
  public static final int SOUTH  = 2;
  public static final int EAST   = 3;
  public static final int WEST   = 4;
  public static final int CENTER = 5;

  // private member variables
  private Vector images;
  private int orientation;
  private int alignment;
  private Dimension dim;

  
  // Constructors
  public ImageRow(Vector new_images, int new_orient, int new_align) {
    images = new_images;
    orientation = new_orient;
    alignment = new_align;
    dim = new Dimension(0,0);
  }


  // newImages replaces the current `images' Vector with `new_images'
  public void newImages(Vector new_images) {
    images = new_images;
  }


  // newAlignment changes the current value of `alignment' to `new_align'
  public void newAlignment(int new_align) {
    alignment = new_align;
  }


  // preferredSize is called by a layout manager when it wants to
  // know how big we'd like to be
  public Dimension getPreferredSize() {
    getMinimumSize();
    dim.width += 4;
    dim.height += 4;
    return dim;
  }


  // minimumSize is called when the layout manager wants to know
  // the minimum amount of space we need
  public Dimension getMinimumSize() {
    Image image;
    dim = new Dimension(0, 0);
    for (int i = 0; i < images.size(); ++i) {
      image = (Image)images.elementAt(i);
      if (orientation == HORIZONTAL) {
	dim.width += image.getWidth(getParent());
	if (dim.height < image.getHeight(getParent()))
	  dim.height = image.getHeight(getParent());
      } else {
	dim.height += image.getHeight(getParent());
	if (dim.width < image.getWidth(getParent()))
	  dim.width = image.getWidth(getParent());
      }
    }
    return dim;
  }


  // paint draws the images
  public void paint(Graphics g) {
    Dimension d = getSize();
    Image image;
    Image buffer = super.createImage(d.width, d.height);
    Graphics b = buffer.getGraphics();

    int 
      x = 2,
      y = 2;

    // Draw a cool border
    b.setColor(getBackground());
    b.fillRect(0, 0, d.width - 1, d.height - 1);
    b.setColor(Color.lightGray);
    b.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
    b.draw3DRect(1, 1, d.width - 3, d.height - 3, true);

    // draw the images
    for (int i = 0; i < images.size(); ++i) {
      image = (Image)images.elementAt(i);

      // set the location pointer
      switch (alignment) {
      case NORTH:  y = 2;                                        break;
      case SOUTH:  y = d.height - image.getHeight(getParent());  break;
      case EAST:   x = d.width - image.getWidth(getParent());    break;
      case WEST:   x = 2;                                        break;
      default:
	if (orientation == HORIZONTAL) 
	  y = (d.height - image.getHeight(getParent())) / 2;
	else
	  x = (d.width - image.getWidth(getParent())) / 2;
	break;
      }

      b.drawImage(image, x, y, this);
      x += image.getWidth(getParent());
      y += image.getHeight(getParent());
    }
    //buffer.flush();
    g.drawImage(buffer, 0, 0, this);
  }
}
