package my.components;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class PButton extends Component implements MouseListener {

  // Button label and information
  private String[] lines = new String[1];
  public Dimension margin = new Dimension(2, 2);
  private Dimension forced;
  private int num_lines;
  private int line_height;
  private int line_ascent;
  private int[] line_widths;
  private int max_width;
  protected Image image = null;

  // Mouse button state variables
  public int button_pressed = 0;
  public int number_pressed = 0;

  // Button colors
  protected Color top_left = TOP_LEFT;
  protected Color bottom_right = BOTTOM_RIGHT;
  protected Color back_color;
  protected Color fore_color;

  // Boolean variables
  protected boolean button_up = true;
  protected boolean mouse_inside = false;
  protected boolean square = false;
  protected boolean dimensions = false;
  public boolean highlight = false;
  public boolean text = true;
  public boolean flatten = false;

  // Constants
  protected static final Color TOP_LEFT = new Color(224, 224, 224);
  protected static final Color BOTTOM_RIGHT = new Color(64, 64, 64);
  public static final int LEFT = 1;
  public static final int CENTER = 2;
  public static final int RIGHT = 4;

  private Vector action_listeners = new Vector(1,1);

  // Constructors
  public PButton(String new_label, Font new_font, int padx, int pady) {
    newLabel(new_label);
    super.setFont(new_font);
    margin.width = padx;
    margin.height = pady;
    addMouseListener(this);
  }

  public PButton(String new_label, int padx, int pady) {
    newLabel(new_label);
    margin.width = padx;
    margin.height = pady;
    addMouseListener(this);
  }

  public PButton(String new_label, Font new_font) {
    newLabel(new_label);
    super.setFont(new_font);
    addMouseListener(this);
  }

  public PButton(String new_label) {
    newLabel(new_label);
    addMouseListener(this);
  }

  public PButton(Image new_image, int padx, int pady) {
    setImage(new_image);
    margin.width = padx;
    margin.height = pady;
    addMouseListener(this);
  }

  public PButton(Image new_image) {
    setImage(new_image);
    addMouseListener(this);
  }


  // This method breaks a specified label up into an array of lines.
  // It uses the StringTokenizer utility class.
  private void newLabel(String new_label) {
    StringTokenizer t = new StringTokenizer(new_label, "\n");
    num_lines = t.countTokens();
    lines = new String[num_lines];
    line_widths = new int[num_lines];
    for (int i = 0; i < num_lines; ++i)
      lines[i] = t.nextToken();
  }


  // This method figures out how large the font is, how wide each
  // line of the label is, and how wide the widest line is.
  public void measure() {
    FontMetrics fm = getFontMetrics(getFont());

    if (fm == null) return;

    line_height = fm.getHeight();
    line_ascent = fm.getAscent();
    max_width = 0;
    for (int i = 0; i < num_lines; ++i) {
      line_widths[i] = fm.stringWidth(lines[i]);
      if (line_widths[i] > max_width) max_width = line_widths[i];
    }
  }


  // This method attempts to create a "brighter" version of the given color
  protected static Color brighter(Color old, float percent) {
    int change = (int)(255f * percent);
    int r = old.getRed() + change, 
      g = old.getGreen() + change, 
      b = old.getBlue() + change;

    // Check the variable bounds
    if (r > 255) r = 255;
    if (g > 255) g = 255;
    if (b > 255) b = 255;

    return new Color(r, g, b);
  }


  // This method attempts to create a "brighter" version of the given color
  protected static Color darker(Color old, float percent) {
    int change = (int)(255f * percent);
    int r = old.getRed() - change, 
      g = old.getGreen() - change, 
      b = old.getBlue() - change;

    // Check the variable bounds
    if (r < 0) r = 0;
    if (g < 0) g = 0;
    if (b < 0) b = 0;

    return new Color(r, g, b);
  }


  /*
   *  getLabel returns the first line in the label.  This is done to 
   *  maintain a certain level of compatibility with the Java Button.
   */
  public String getLabel() {
    return lines[0];
  }


  /*
   *  getMLabel returns the full label for the PButton
   */
  public String[] getPLabel() {
    return lines;
  }


  // This method gives the user the power to change
  // the label.  The button then recalculates the
  // label measurements.
  public void setLabel(String new_label) {
    text = true;
    newLabel(new_label);
    measure();
  }

  public void setImage(Image new_image) {
    text = false;
    image = new_image;
  }

  // This method changes the background color (text)
  // for the button
  public void setBackground(Color new_back) {
    back_color = new_back;
    super.setBackground(new_back);
  }


  // This method changes the foreground color (text)
  // for the button
  public void setForeground(Color new_fore) {
    fore_color = new_fore;
    super.setForeground(new_fore);
  }


  // This method changes the font and redraws the button
  public void setFont(Font new_font) {
    super.setFont(new_font);
    measure();
  }


  // This method insures that the dimensions of the button are equal
  public void forceSquare() {
    dimensions = false;
    square = true;
  }


  // This method allows the user to force the dimensions of the button
  public void forceDimension(Dimension new_dim) {
    dimensions = true;
    square = false;
    forced = new_dim;
  }


  /*
   *  addActionListener adds a class to the list of action listeners to the 
   *  PButton object
   */
  public synchronized void addActionListener(ActionListener l) {
    action_listeners.addElement(l);
  }


  /*
   *  removeActionListener removes the given actionListener from the list 
   *  of action listeners.
   */
  public synchronized void removeActionListener(ActionListener l) {
    action_listeners.removeElement(l);
  }


  /*
   *  actionPerformed is called whenever an action is performed on this object
   */
  public void actionPerformed(ActionEvent e) {
    for (int i = 0; i < action_listeners.size(); ++i) {
      ((ActionListener)(action_listeners.elementAt(i))).actionPerformed(e);
    }
  }


  // This method tells the caller if the mouse is inside of the button
  public boolean mouseInside() {
    return mouse_inside;
  }

  // This method tells the caller if the left mouse button was pressed
  public boolean mouseLeft() {
    return ((button_pressed & LEFT) != 0);
  }

  // This method tells the caller if the center mouse button was pressed
  public boolean mouseCenter() {
    return ((button_pressed & CENTER) != 0);
  }

  // This method tells the caller if the right mouse button was pressed
  public boolean mouseRight() {
    return ((button_pressed & RIGHT) != 0);
  }


  // This method is invoked when the user presses the mouse button
  // over the button.  It redraws the button so that it looks like
  // it is recessed.
  public void mousePressed(MouseEvent e) {

    // Keep track of which mouse buttons are pressed
    if (number_pressed == 0)
      button_pressed = 0;

    if (((e.getModifiers() & InputEvent.BUTTON3_MASK) > 0) && 
	((button_pressed & RIGHT) == 0))
      button_pressed |= RIGHT;
    else if (((e.getModifiers() & InputEvent.BUTTON2_MASK) > 0) && 
	     ((button_pressed & CENTER) == 0))
      button_pressed |= CENTER;
    else
      button_pressed |= LEFT;

    // else we clicked n buttons, released n-1 buttons, and clicked a button (n > 1)
    ++number_pressed;
    button_up = false;
    if (!flatten)
      paint(getGraphics());
  }


  // This methods is called when the user releases the mouse button
  // after pressing down on this button.  It redraws the button to
  // its original appearance.
  public void mouseReleased(MouseEvent e) {

    // Since I cannot keep track of buttons that are released, as they are 
    // released, I will maintain a count of the number of buttons pressed
    --number_pressed;

    button_up = true;
    if (!flatten)
      paint(getGraphics());
  }


  // This method is called when the user moves the mouse inside of
  // the button.  If enabled, the button highlights itself.
  public void mouseEntered(MouseEvent e) {
    mouse_inside = true;
    if (highlight)
      paint(getGraphics());
  }


  // This method is called when the user moves the mouse outside of
  // the button.  if enabled, the button unhighlights itself.
  public void mouseExited(MouseEvent e) {
    mouse_inside = false;
    if (highlight)
      paint(getGraphics());
  }

  public void mouseClicked(MouseEvent e) {
    this.actionPerformed(new ActionEvent(this, this.hashCode(), this.getLabel(), 
					 ActionEvent.ACTION_PERFORMED));
  }

  // This method is invoked after our Canvas if first created
  // but before it can actually be displayed.  After we've
  // invoked our superclass's addNotify() method, we have font
  // metrics and can successfully call measure() to figure out
  // how big the label is
  public void addNotify() {
    super.addNotify();
    back_color = super.getBackground();
    fore_color = super.getForeground();
    measure();                  // This is the proper time to call measure()
  }


  // This method is called by a layout manager when it wants to
  // know how big we'd like to be.
  public Dimension getPreferredSize() {
    Dimension dim = new Dimension(0,0);

    if (text) {
      dim.width = max_width + 2 * margin.width + 4;
      dim.height = num_lines * line_height + 2 * margin.height + 4;
    } else {
      dim.width = image.getWidth(getParent()) + 2 * margin.width + 4;
      dim.height = image.getHeight(getParent()) + 2 * margin.height + 4;
    }

    if (square) {
      if (dim.width > dim.height) dim.height = dim.width;
      else dim.width = dim.height;
    } 
    else if (dimensions)
      dim = forced;

    return dim;
  }


  // This method is called when the layout manager wants to know
  // the bare minimum amount of space we need to get by.
  public Dimension getMinimumSize() {
    Dimension dim = new Dimension(0,0);

    if (text) {
      dim.width = max_width + 4;
      dim.height = num_lines * line_height + 4;
    } else {
      dim.width = image.getWidth(getParent()) + 4;
      dim.height = image.getHeight(getParent()) + 4;
    }

    if (square) {
      if (dim.width > dim.height) dim.height = dim.width;
      else dim.width = dim.height;
    } 
    else if (dimensions)
      dim = forced;

    return dim;
  }


  // This method draws the button.
  // Note that it handles the margins and the 
  // alignment, as well as the color and font.
  // It also returns the color and font to their
  // original state
  public void paint(Graphics g) {
    Dimension d = getSize();
    Image buffer = super.createImage(d.width, d.height);
    Graphics b = buffer.getGraphics();

    // Draw the button's background
    if (button_up || flatten)
      b.setColor(back_color);
    else
      b.setColor(darker(back_color, 0.10f));
    if (mouse_inside && highlight) b.setColor(brighter(b.getColor(), 0.10f));
    b.fillRect(1, 1, d.width-3, d.height-3);

    // Draw the contents of the button
    if (text) {
      // Setup the (x,y) for the label
      int x, y = line_ascent + (d.height - num_lines * line_height) / 2;
      if (!button_up && !flatten) y += 2;

      // Draw the label
      b.setColor(fore_color);
      b.setFont(getFont());
      for (int i = 0; i < num_lines; ++i, y += line_height) {
	x = (d.width - line_widths[i]) / 2;
	if (!button_up && !flatten) x += 2;
	b.drawString(lines[i], x, y);
      }
    } else {

      // Setup the x, y
      int x = 0, y = 0;
      if (!button_up && !flatten) { x += 1;  y += 1; } 
      b.drawImage(image, (d.width - image.getWidth(getParent())) / 2 + x, 
		  (d.height - image.getHeight(getParent())) / 2 + y, this);
    }

    // Draw the button's border
    if (flatten) {
      b.setColor(back_color);
      b.draw3DRect(0, 0, d.width-1, d.height-1, false);
    } else {
      if (button_up)
	b.setColor(back_color);
      else
	b.setColor(darker(back_color, 0.10f));
      if (mouse_inside && highlight) b.setColor(brighter(b.getColor(), 0.10f));

      b.draw3DRect(0, 0, d.width-1, d.height-1, button_up);
      if (button_up)
	b.draw3DRect(0, 0, d.width-2, d.height-2, button_up);
      else
	b.draw3DRect(1, 1, d.width-2, d.height-2, button_up);
    }
    buffer.flush();
    g.drawImage(buffer, 0, 0, getParent());
  }
}
